// Undo_Redo_DemoDoc.cpp : implementation of the CUndo_Redo_DemoDoc class
// As part of the Undo_Redo_Demo application
// By Tom Morris of trilobiteWorks. December 2002
// Use freely...

#include "stdafx.h"
#include "Undo_Redo_Demo.h"
#include "Undo_Redo_DemoDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//	this flag enables the ability to Undo all the way back
//	to the origination of the document -- regardless of the
//	number of sessions involved. Drawback = large file size.
//	If you only need CURRENT SESSION Undos, comment out
//	the #define INFINITE_UNDO statement below.
#define	INFINITE_UNDO


// CUndo_Redo_DemoDoc

IMPLEMENT_DYNCREATE(CUndo_Redo_DemoDoc, CDocument)

BEGIN_MESSAGE_MAP(CUndo_Redo_DemoDoc, CDocument)
END_MESSAGE_MAP()


// CUndo_Redo_DemoDoc construction/destruction

CUndo_Redo_DemoDoc::CUndo_Redo_DemoDoc()
{
	m_bUndosAreAvailable = FALSE;
	m_bRedosAreAvailable = FALSE;
}

CUndo_Redo_DemoDoc::~CUndo_Redo_DemoDoc()
{
}


/////////////////////////////////////////////////////////////////////////////
//	OnNewDocument
//	
/////////////////////////////////////////////////////////////////////////////
BOOL CUndo_Redo_DemoDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	//	Because this is a Single Document Interface (SDI), this document here
	//	gets used by incoming, new or loaded documents. So we need to
	//	clear out the UpdateMgr and start with a fresh set of DocVars.
	//	These steps are not necessary in Multi-Document Interface apps
	//	because each newly loaded document is unique unto itself.
	m_CurrentDocVars = m_UpdateMgr.DocVarsReset();
	DocVarsUpdate(m_CurrentDocVars);
	//	End SDI DocVars Reinitialization

	
	UpdateAllViews(NULL);

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
//	Serialize
//	
/////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::Serialize(CArchive& ar)
{
#ifdef	INFINITE_UNDO
	//	Infinite Undo	********************************************
	//	this demo saves the whole list of DocVars
	//	By doing this, it is possible to open up the document at
	//	any stage of its development and UNDO it all the way to its
	//	origination. if you don't want to do this, comment out the
	//	lines below.
	CtObjectList <CDocVars, CDocVars&>saveDocVarsList;
	CtObjectList <CDocVars, CDocVars&>loadDocVarsList;

	//	We are going to serialize the list of all DocVars so we can still
	//	Undo all the way to the beginning if we want to.
	//	Get the latest list of DocVars from UpdateMgr.
	saveDocVarsList = m_UpdateMgr.m_lDocVarsList;
	//	End Infinite Undo	****************************************
#endif

	if (ar.IsStoring())
	{
		m_CurrentDocVars.Serialize(ar);

#ifdef	INFINITE_UNDO
		//	Infinite Undo	********************************************
		//	this demo saves the whole list of DocVars
		//	By doing this, it is possible to open up the document at
		//	any stage of its development and UNDO it all the way to its
		//	origination. if you don't want to do this, comment out the
		//	line below.
		saveDocVarsList.Serialize(ar);	
		//	End Infinite Undo	****************************************
#endif

	}
	else
	{
		//	Because this is a Single Document Interface (SDI), this document here
		//	gets used by incoming, new or loaded documents. So we need to
		//	clear out the UpdateMgr and start with a fresh set of DocVars.
		//	These steps are not necessary in Multi-Document Interface apps
		//	because each newly loaded document is unique unto itself.
		m_UpdateMgr.DocVarsReset();
		//	End SDI DocVars Reinitialization

		m_CurrentDocVars.Serialize(ar);
		DocVarsUpdate(m_CurrentDocVars);
		
#ifdef	INFINITE_UNDO
		//	Infinite Undo	********************************************
		//	this demo saves the whole list of DocVars
		//	By doing this, it is possible to open up the document at
		//	any stage of its development and UNDO it all the way to its
		//	origination. if you don't want to do this, comment out the
		//	lines below.
		loadDocVarsList.Serialize(ar);	//	load the list of DocVars (history)

		//	Update the list in UpdateMgr.
		//	All previously-stored docvars and plugin objects are restored
		m_UpdateMgr.m_lDocVarsList = loadDocVarsList;
		//	Set the current DocVars at list end
		m_UpdateMgr.m_posCurrentPosition = 	m_UpdateMgr.m_lDocVarsList.GetTailPosition();
		//	set the currentDocVars to list end
		m_CurrentDocVars = m_UpdateMgr.m_lDocVarsList.GetTail();
		//	End Infinite Undo	****************************************
#endif
		
		m_UpdateMgr.UpdateUndoRedoAvailability();

	}
	UpdateAllViews(NULL);
}


// CUndo_Redo_DemoDoc diagnostics

#ifdef _DEBUG
/////////////////////////////////////////////////////////////////////////////
//	AssertValid
//	
/////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::AssertValid() const
{
	CDocument::AssertValid();
}


/////////////////////////////////////////////////////////////////////////////
//	Dump
//	
/////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
//	SetUndoAvailability
//	
/////////////////////////////////////////////////////////////////////////////
void	CUndo_Redo_DemoDoc::SetUndoAvailability(BOOL bAvail)
{
	m_bUndosAreAvailable = bAvail;
}


/////////////////////////////////////////////////////////////////////////////
//	SetRedoAvailability
//	
/////////////////////////////////////////////////////////////////////////////
void	CUndo_Redo_DemoDoc::SetRedoAvailability(BOOL bAvail)
{
	m_bRedosAreAvailable = bAvail;
}


/////////////////////////////////////////////////////////////////////////////
//	WeHaveUndosLeft
//	
/////////////////////////////////////////////////////////////////////////////
BOOL	CUndo_Redo_DemoDoc::WeHaveUndosLeft()
{
	return m_bUndosAreAvailable;
}


/////////////////////////////////////////////////////////////////////////////
//	WeHaveRedosLeft
//	
/////////////////////////////////////////////////////////////////////////////
BOOL	CUndo_Redo_DemoDoc::WeHaveRedosLeft()
{
	return m_bRedosAreAvailable;
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsCreateUniqueCopy
//	
///////////////////////////////////////////////////////////////////////////////
CDocVars	CUndo_Redo_DemoDoc::DocVarsCreateUniqueCopy(CDocVars theseDocVars)
{
	/*	This function creates new and unique copies of a CDocVars variables.
	This is necessary in case you have pointers because just doing:
	theseDocVars = XXXDocVars;
	just creates new sets of pointers to the same old objects. It doesn't create 
	new data instances.

	The problem with the = approach is that if you edit a pointer variable in one DocVars
	instance, those changes are reflected in all other DocVars instances possessing
	the particular pointer variable. This kinda defeats the purpose of Undo / Redo.

	Although no DocVars pointers are included in this demo, please keep this in mind
	if you need to use pointers in your DocVars.

	You would need to retrieve the values for your pointer variables and store them
	here temporarily. Then after you have created a new copy of theseDocVars, create
	new instances of your pointer variables and fill in the parameters from the locally
	stored set.
	*/
	
	CDocVars	uniqueDocVars;						
	uniqueDocVars = theseDocVars;

	return uniqueDocVars;
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsUndo
//	
///////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::DocVarsUndo()
{
	m_CurrentDocVars = m_UpdateMgr.DocVarsUndo();
	UpdateAllViews(NULL);
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsRedo
//	
///////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::DocVarsRedo()
{
	m_CurrentDocVars = m_UpdateMgr.DocVarsRedo();
	UpdateAllViews(NULL);
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsUpdate
//	
///////////////////////////////////////////////////////////////////////////////
void CUndo_Redo_DemoDoc::DocVarsUpdate(CDocVars DocVars)
{
	/*	before we do this, we need to make a unique duplicate of the sent DocVars.
		Once we have our unique duplicate, we can update the UpdateMgr with it.
	*/
	CDocVars	thisFunctionDocVars;	

	//	create the unique new docvars
	thisFunctionDocVars = DocVarsCreateUniqueCopy(DocVars);

	//	inform UpdateMgr and get the docVars at the end of it's list
	m_CurrentDocVars = m_UpdateMgr.DocVarsUpdate(thisFunctionDocVars);

	UpdateAllViews(NULL);

	SetModifiedFlag();
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsGetByValue
//	
///////////////////////////////////////////////////////////////////////////////
CDocVars	CUndo_Redo_DemoDoc::DocVarsGetByValue()
{
	//	returns a new instance of the DocVars that reflects the values
	//	of the m_CurrentDocVars. Use this to setup a local copy of the DocVars
	//	when you need to make a change to one of the DocVars variables.

	return m_CurrentDocVars;
}


///////////////////////////////////////////////////////////////////////////////
//	DocVarsGetPointer
//	
///////////////////////////////////////////////////////////////////////////////
CDocVars	*CUndo_Redo_DemoDoc::DocVarsGetPointer()
{
	//	*********	IMPORTANT	*************
	//	NEVER NEVER NEVER NEVER NEVER EVER use this for the purpose of
	//	changing the value of a DocVars variable. 
	//	Only use this if you need to know the value of a DocVars variable.

	return &m_CurrentDocVars;
}


