// UpdateMgr.cpp : implementation file
// As part of the Undo_Redo_Demo application
// By Tom Morris of trilobiteWorks. December 2002
// Use freely...

#include "stdafx.h"
#include "Undo_Redo_DemoDoc.h"
#include "UpdateMgr.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUpdateMgr

CUpdateMgr::CUpdateMgr()
{
	m_posCurrentPosition = NULL;
}

CUpdateMgr::~CUpdateMgr()
{


}


/////////////////////////////////////////////////////////////////////////////
//	DocVarsReset
//	
/////////////////////////////////////////////////////////////////////////////
CDocVars	CUpdateMgr::DocVarsReset()
{
	CDocVars	freshNewDocVars;

	m_lDocVarsList.RemoveAll();
	m_posCurrentPosition = NULL;
	m_CurrentDocVars = freshNewDocVars;

	UpdateUndoRedoAvailability();

	return m_CurrentDocVars;
}


/////////////////////////////////////////////////////////////////////////////
//	DocVarsUpdate
//	
/////////////////////////////////////////////////////////////////////////////
CDocVars&	CUpdateMgr::DocVarsUpdate(CDocVars	DocVars)
{
	if (m_posCurrentPosition)
	{
		m_posCurrentPosition = m_lDocVarsList.InsertAfter(m_posCurrentPosition, m_CurrentDocVars);
		m_lDocVarsList.GetAt(m_posCurrentPosition) = DocVars;
		m_CurrentDocVars = m_lDocVarsList.GetAt(m_posCurrentPosition);

		//	if we are updating while in the middle of the list, we have to 
		//	remove all trailing items in the list. Otherwise a Redo request
		//	will "advance" us to a previous state... CONFUSING!	
		//	Therefore, all edits and updates from mid-list will force
		//	a tail-trimming and loss of access to trimmed list items
		POSITION tempPos = m_lDocVarsList.GetTailPosition();
		while (tempPos != m_posCurrentPosition)
		{
			m_lDocVarsList.RemoveTail();
			tempPos = m_lDocVarsList.GetTailPosition();
		}
		UpdateUndoRedoAvailability();
		return m_CurrentDocVars;
	}
	else if (!m_posCurrentPosition)
	{
		m_posCurrentPosition = m_lDocVarsList.AddTail(m_CurrentDocVars);
		m_lDocVarsList.GetTail() = DocVars;
		m_CurrentDocVars = m_lDocVarsList.GetTail();
		UpdateUndoRedoAvailability();
		return m_CurrentDocVars;
	}

	UpdateUndoRedoAvailability();
	return  m_CurrentDocVars;
}


/////////////////////////////////////////////////////////////////////////////
//	DocVarsUndo
//	
/////////////////////////////////////////////////////////////////////////////
CDocVars		CUpdateMgr::DocVarsUndo()
{
	if (m_posCurrentPosition != m_lDocVarsList.GetHeadPosition())
	{
		m_lDocVarsList.GetPrev(m_posCurrentPosition);
		m_CurrentDocVars = m_lDocVarsList.GetAt(m_posCurrentPosition);
		UpdateUndoRedoAvailability();
		return m_CurrentDocVars;
	}
	else if (m_posCurrentPosition == m_lDocVarsList.GetHeadPosition())
	{
		m_CurrentDocVars = m_lDocVarsList.GetHead();
	}
	UpdateUndoRedoAvailability();
	return m_CurrentDocVars;
}


/////////////////////////////////////////////////////////////////////////////
//	DocVarsRedo
//	
/////////////////////////////////////////////////////////////////////////////
CDocVars		CUpdateMgr::DocVarsRedo()
{
	if (m_posCurrentPosition != m_lDocVarsList.GetTailPosition())
	{
		m_lDocVarsList.GetNext(m_posCurrentPosition);
		m_CurrentDocVars = m_lDocVarsList.GetAt(m_posCurrentPosition);
		UpdateUndoRedoAvailability();
		return m_CurrentDocVars;
	}
	else if (m_posCurrentPosition == m_lDocVarsList.GetTailPosition())
	{
		m_CurrentDocVars = m_lDocVarsList.GetTail();
	}
	UpdateUndoRedoAvailability();
	return m_CurrentDocVars;
}


/////////////////////////////////////////////////////////////////////////////
//	UpdateUndoRedoAvailability
//	
/////////////////////////////////////////////////////////////////////////////
bool CUpdateMgr::UpdateUndoRedoAvailability()
{
	CUndo_Redo_DemoDoc *pDoc = NULL;
	CMainFrame *pMainFrame = NULL;
	pMainFrame = (CMainFrame*)AfxGetApp()->GetMainWnd();
	if (pMainFrame)
	{
		pDoc = (CUndo_Redo_DemoDoc*)pMainFrame->GetActiveDocument();

		if (pDoc)
		{
			if ((m_lDocVarsList.GetCount() > 1) && (m_posCurrentPosition != m_lDocVarsList.GetTailPosition()))
				pDoc->SetRedoAvailability(true);
			else
				pDoc->SetRedoAvailability(false);

			if ((m_lDocVarsList.GetCount() > 1) && (m_posCurrentPosition != m_lDocVarsList.GetHeadPosition()))
				pDoc->SetUndoAvailability(true);
			else
				pDoc->SetUndoAvailability(false);

			return true;
		}
		return false;
	}
	return false;
}

